#!/bin/bash

ACTION="$1"
FILENAME="/etc/apt/sources.list.d/lipoffline.list"
ACTIVATED=`test -e "$FILENAME"`
DIST_CODENAME="trusty"
DIST_VERSION="14.04"


function install_repo()
{
	echo "# offline repository of the linux install party
deb [ trusted=yes ] file://$1/archives $DIST_CODENAME lip" > "$FILENAME"
}

function uninstall_repo()
{
	rm -f "$FILENAME"
}

case $ACTION in
	"on")
		if [ -z "$2" ]; then
			echo "No repository location specified!"
			exit 2
		fi
		install_repo "$2"
		apt-get update
		echo "Repository is now active, remember to call \"$0 off\" to deactivate it when you are done."
		;;
	"off")
		uninstall_repo
		apt-get update
		echo "Repository is now inactive."
		;;
	"check")
		echo "State: $ACTIVATED"
		;;
	*)
		echo "Unknown action $ACTION"
		exit 1
		;;
esac
